function NIRIS_raw2Stokes, date, calib, makednf, flatndark, align, surfix, prefixi, prefixo
;+
; IRIM_raw2Stokes.pro
;
; :Author: KWANGSU
; This program produces 4 Stokes signals from dual beam observation.
;- Major modification:
;  / / : Changed orientation of dual beam from horizontal split to vertical split. Ver. 1.5
;  07/10/2014: Changed orientation of the polarization beam splitter so that dual beam changes signs. : Ver. 1.6
;  07/16/2014: Changed code in compensation of phase.:Ver. 2.0
;
; The output data type:
; img[1024/binx, 512/biny, 13, 4]
; the 4th elements are
; 0: intensity profile (not normalized to the continuum)
; 1: Stokes Q (phase shift corrected, ripple readout not corrected)
; 2: Stokes U
; 3: Stokes V


;makednf: Create dark and flat
;flatndark: Perform dark subtraction and flat fielding from raw images
;align: Align frames and get results

; manf: if flat files should be chosen manually since flat data are in a form of image data
;        in this case, users must copy the flat files into the 'calib' folder of processing disk
;        to see what files should be copied, please see the log files in the raw data.


;PARAMETERS
platform='win'
set_plot, platform

if platform eq 'win' then dl='\' else dl='/'

patho=prefixo+date+dl+surfix
file_mkdir, patho
;copy position 0: no copy, 1: copy w/ovewrite, 2: copy while taking data
copy_position=1
if platform eq 'win' then t_comp_path='F:\N_temp' else stop
running_erase=1
dfcomp=1
load_shifts=1
file_mkdir, t_comp_path

pathi=prefixi+date+dl+surfix

if date lt '20150101' then begin
  c2048=0 & chipsz=1024
endif else begin
  c2048=1 & chipsz=2048
endelse
if date ge '20150501' then imrot=1 else imrot=0

;------------------------------------------------
if makednf then begin

  year=strmid(date,0,4)
  month=strmid(date,4,2)
  day=strmid(date,6,2)
  case month of
  '01':monc='jan'
  '02':monc='feb'
  '03':monc='mar'
  '04':monc='apr'
  '05':monc='may'
  '06':monc='jun'
  '07':monc='jul'
  '08':monc='aug'
  '09':monc='sep'
  '10':monc='oct'
  '11':monc='nov'
  '12':monc='dec'

  endcase


  testindex=file_search(pathi+'*.*')

  if testindex[0] eq '' then begin
    print, 'No file found.'
    stop
  endif

  if calib then filelst=file_search(pathi+'calib'+dl+'*_c_*.fts*') else filelst=file_search(pathi+'*_i_*.fts*')
  help, filelst

      ;SIDE JOB: COPY POSITION files ____________
  if copy_position ne 0 then begin
    print, 'Copying position files...'

    if calib then pindex=file_search(pathi+'calib'+dl+'frtimes_*.txt') $
      else pindex=file_search(pathi+'frtimes_*.txt')
  
    if n_elements(pindex) ge 1 then begin
      file_mkdir, patho+'position'
      if copy_position eq 1 then file_copy, pindex, patho+'position', /overwrite $
        else file_copy, pindex[0:nm1(pindex)-1], patho+'position', /overwrite
    print, 'Done cpy position'
  
    endif
  ;END COPY POSITION

  endif

  ;check wavelength number
  nf_array=intarr(n_elements(filelst))
  ny_array=intarr(n_elements(filelst), 4)

  print, 'Checking median number of frames...'
  for i=0, nm1(filelst) do begin
    headers=headfits(filelst[i], /sil)
    if fxpar(headers, 'NAXIS') ne 0 then begin
    nf_array[i]=fxpar(headers, 'NAXIS3')
    ny_array[i,0]=fxpar(headers, 'Y0')
    ny_array[i,1]=fxpar(headers, 'Y1')
    ny_array[i,2]=fxpar(headers, 'NAXIS1')
    ny_array[i,3]=fxpar(headers, 'NAXIS2')
    endif else begin
      headers=headfits(filelst[i], /exten, /sil)
      nf_array[i]=fxpar(headers, 'ZNAXIS3')
      ny_array[i,0]=fxpar(headers, 'Y0')
      ny_array[i,1]=fxpar(headers, 'Y1')
      ny_array[i,2]=fxpar(headers, 'ZNAXIS1')
      ny_array[i,3]=fxpar(headers, 'ZNAXIS2')
    endelse
  endfor

  nf=fix(median(nf_array))
  nfpw=18
  nw=nf/nfpw
  if nf mod nfpw ne 0 then stop
  print, 'nw=', nw

  nf=nfpw*nw

  Y0=fix(median(ny_array[*,0]))
  Y1=fix(median(ny_array[*,1]))
  NX=fix(median(ny_array[*,2]))
  NY=fix(median(ny_array[*,3]))
  ;NY=Y1-Y0+1

 if NX eq 2048 and NY eq 1024 then ROIs=1 else ROIs=0

  if NY ne 1 then begin
    save, Y0, Y1, NX, NY, ROIs, filename=patho+date+'_NY.sav'

  endif else begin

      Y0=0
      Y1=chipsz-1
      NY=chipsz

  endelse
        print, 'Y0=', Y0, ' Y1=', Y1, 'NX=', NX, ' NY=', NY
  save, nf, nfpw, nw, filelst, filename=patho+date+'_params.sav'

endif
;-----------------------------------------------------
if align then begin
;TO DETERMINE A REGEON OF INTEREST FOR ALIGNMENT.
;OTHERWISE, THE ALIGNMENT WILL BE DONE WITH PRESET ROIs.
ROI0=0; Alignment of frames in a burst
ROI1=0; Alignment of dual beams
 ;calib overrides ROI0 and ROI1
 ;with calib, threshold to mask bad pixels will not work

align_dest=1
;set rotation degree for align
;drot=2.2/(-2.)
;if date ge '20140601' then drot=3.5/(-2.)
;if date ge '20140701' then drot=2.8/(-2.)
;if date ge '20140801' then drot=-2.65/(-2.)

;print, 'Image derotation by '+string(drot)+' degree was applied.'

if date ge '20140601' and date le '20140731' then rm_bk=1 else rm_bk=0

endif
;-------------------------------------------------------

;__________________________________________
if makednf ne 1 then begin
    restore, filename=patho+date+'_params.sav'
    ny_test=file_test(patho+date+'_NY.sav')
    if ny_test then begin
       restore, filename=patho+date+'_NY.sav'
    endif else begin
        Y0=0
        Y1=chipsz-1
        NX=chipsz
        NY=chipsz

    endelse
endif
is=0;Starting burst (0 is the first)
ie=nm1(filelst);;Ending burst
;__________________________________________


if calib then begin
  if c2048 then binx=64 else binx=512
  biny=2
endif else begin
   binx=2
  biny=2
  if date eq '20150616' or date eq '20150618' or date eq '20150622' or date eq '20150707' then begin & binx=1 & biny=1 & endif
 
endelse




;_____________CREATE DARK AND FLAT_____________________________________

if( makednf ) then begin



  flatindex=file_search(pathi+'calib'+dl+'bbso_irim*_f_*.fts*')
  darkindex=file_search(pathi+'calib'+dl+'bbso_irim*_d_*.fts*')



  ;Make dark (dark should be one file)
  
  if strmid(darkindex[0], strlen(darkindex[0])-2, 2) eq 'fz' then begin
    fout=NIRIS_decomp(darkindex[0], t_comp_path)
    if fout ne '' then darkindex[0]=fout
  endif
  
  
  darkraw=readfits(darkindex[0], dh, /sil)
  
  dark=intarr(NX, NY)
  dark=fix(median(darkraw, dimension=3))
  !null=temporary(darkraw)
  file_mkdir, patho+'calib'
  writefits, patho+'calib'+dl+'dark.fts', dark, dh

  ;Make flat
  flatarray=intarr(NX,NY,nw)
  itemp=intarr(NX,NY,nf)
  
  for i=0, n_elements(flatindex)-1 do begin
    if strmid(flatindex[i], strlen(flatindex[i])-2, 2) eq 'fz' then begin
      fout=NIRIS_decomp(flatindex[i], t_comp_path)
      if fout ne '' then flatindex[i]=fout else stop
    endif
    
    if (i eq 0) then itemp+=(readfits(flatindex[i], fh, /silent)/n_elements(flatindex)) $
      else itemp+=(readfits(flatindex[i],/silent)/n_elements(flatindex))
  endfor

  for i=0, nw-1 do flatarray[*,*,i]=total(itemp[*,*,i*nfpw:i*nfpw+15], 3)/16

  writefits, patho+'calib'+dl+'flat.fts', flatarray, fh
  !null=temporary(itemp)
  
  !null=temporary(flatarray)
  !null=temporary(dark)

  print, 'Flat & dark made...'
endif

;_____________________________________________________________


;_____________PROCESS DARK & FLAT_____________________________

if (flatndark) then begin


  file_mkdir, patho+'df'
  index=filelst

  dark=readfits(patho+'calib'+dl+'dark.fts', /silent)
  flat=readfits(patho+'calib'+dl+'flat.fts', /silent)

  fdfac=intarr(nw)

  fd=intarr(NX,NY,nw)
  for k=0, nw-1 do begin
    fd[*,*,k]=flat[*,*,k]-dark
    if c2048 and ~imrot then fdfac[k]=median(fd[800:1200,*,k]) 
    if c2048 and imrot and ~ROIs then fdfac[k]=median(fd[*,800:1200,k])
    if c2048 and imrot and ROIs then fdfac[k]=median(fd[*,*,k])

    if ~c2048 then fdfac[k]=median(fd[*,*,k])  
  endfor

  ;Look for bad pixels switch to the median value


  if ~calib and ~c2048 then begin
    ss=where(fd le 1d3)
    fd[ss]=median(fd)
  endif

  file_mkdir, patho+'df'
  set_plot, 'win'
  window, 0, xs=NX/binx, ys=NY/biny
  !p.multi=[0,1,3]



  for i=is, ie do begin
    ;MODIFIED
    
    if strmid(index[i], strlen(index[i])-2, 2) eq 'fz' then begin
      
    ftime=strmid(index[i], strlen(index[i])-23, 13)
    ftest=(file_test(patho+'df'+dl+'df_'+ftime+'.fts')) or (file_test(patho+'df'+dl+'df_'+ftime+'.fts.fz'))
    
    if ftest then begin
      print, ftime+' already exists. Skipping...'
      goto, JUMP1
    endif      
      
         
      
      fout=NIRIS_decomp(index[i], t_comp_path)
      if fout ne '' then begin
        index[i]=fout
        comp=1
      endif else begin
        print, 'Error: decompression failed...skipping...'
        goto, JUMP1
      endelse
    endif else comp=0
    
    ftime=strmid(index[i], strlen(index[i])-20, 13)
    ftest=(file_test(patho+'df'+dl+'df_'+ftime+'.fts')) or (file_test(patho+'df'+dl+'df_'+ftime+'.fts.fz'))
    
    if ftest then begin
      print, ftime+' already exists. Skipping...'
      goto, JUMP1
    endif



    
    img=readfits(index[i], header, /silent)
    if fxpar(header, 'NAXIS3') ne nf then begin
      print, 'frame number does not match for'+string(i)+'.'
      goto, JUMP1
    endif


    ;modify header for pixel scale
    pixscale=fxpar(header, 'CDELT1')
    pixscalex=pixscale*binx
    pixscaley=pixscale*biny

    fxaddpar, header, 'CDELT1', pixscalex
    fxaddpar, header, 'CDELT2', pixscaley

    for j=0, nf-1 do img[*,*,j]= fix(float(img[*,*,j]-dark)/fd[*,*,j/nfpw]*fdfac[j/nfpw]*5.);/add*5 for testing


    if ~c2048 and ~calib then begin
      ;masking bad columns
      img[253,*,*]=img[252,*,*] & img[254,*,*]=img[252,*,*] & img[255,*,*]=img[256,*,*]
      img[509,*,*]=img[508,*,*] & img[510,*,*]=img[508,*,*] & img[511,*,*]=img[512,*,*]
      img[765,*,*]=img[764,*,*] & img[766,*,*]=img[764,*,*] & img[767,*,*]=img[768,*,*]
      img[1021,*,*]=img[1020,*,*] & img[1022,*,*]=img[1020,*,*] & img[1023,*,*]=img[1020,*,*]

      ss=where(img[*,*,0] le 1d3)
      for j=0, nf-1 do begin
        imgj=img[*,*,j]
        imgj[ss]=imgj[ss+5]
        img[*,*,j]=imgj
      endfor
      
      imgh=img[*,*,0]
      hotmed=median(imgh)
      hotstd=stddev(imgh)
      kernel=[-0.5, 0.5]
      hotpix=where(convol(float(imgh), kernel) gt 100)
      hotx=hotpix mod n_elements(imgh[*,0])
      hoty=hotpix/n_elements(imgh[0,*])

      for j=0, nf-1 do begin
        imgj=img[*,*,j]
        for ti=0, 1 do for tj=-1, 1 do imgj[hotx+ti, hoty+tj]=(imgj[hotx+5, hoty]+imgj[hotx-5, hoty]+imgj[hotx, hoty+5]+imgj[hotx, hoty-5])/4
  
        if j eq 1 then begin
  	      if c2048 then tv, congrid(bytscl(imgj, hotmed-hotstd, hotmed+hotstd),1024,1024) else tv, bytscl(imgj, hotmed-hotstd, hotmed+hotstd)
   	      write_png, patho+'df'+dl+'df_'+ftime+'.png', bytscl(imgj, hotmed-hotstd, hotmed+hotstd)
        endif
        img[*,*,j]=imgj
      endfor

    endif;chipsz

    if calib then begin
      imgc=intarr(NX/binx,NY/biny, nf)
      for j=0, nf-1 do begin
        if c2048 then begin
          for jj=0, 31 do imgc[jj,*,j]=rebin(median(img[64*jj:64*jj+63,*,j], dimension=1), NY/biny)
        endif else begin
          imgc[0,*,j]=rebin(median(img[30:481,*,j], dimension=1), 512)
          imgc[1,*,j]=rebin(median(img[541:993,*,j], dimension=1), 512)
        endelse
      endfor
    endif

   curt=systime()

   fxaddpar, header, 'CDELT1', 0.1660
   fxaddpar, header, 'CDELT2', 0.1660
   fxaddpar, header, 'XCEN', 'REFER TO BBSO LOG AT http://www.bbso.njit.edu/cgi-bin/Logs'
   fxaddpar, header, 'YCEN', 'SAME AS ABOVE'
   fxaddpar, header, 'INSTRUME', 'NIRIS'
   expt=fxpar(header, 'EXPOSURE')
   framerate=1000./(expt+1.)
   fxaddpar, header, 'FRRATE', framerate
   fxaddpar, header, 'HISTORY', 'Processed dark/flat on '+curt
   fxaddpar, header, 'FRMSWVLN', '16+2(extra buffer)'
   startz=fxpar(header, 'STARTZ')
   endz=fxpar(header, 'ENDZ')
   zinc=fxpar(header, 'ZINCREMT')
   st2wv=0.003163
   fxaddpar, header, 'REFWV', 15648.4983, 'Wavelength at zero step (Angstrom)'
   fxaddpar, header, 'STARTWV',startz*st2wv, 'Starting wavelength from REFWV (Angstrom)'
   fxaddpar, header, 'ENDWV', endz*st2wv, 'Ending wavelength from REFWV (Angstrom)'
   fxaddpar, header, 'INCWV', zinc*st2wv, 'Increment per wavelength (Angstrom)'
   fxaddpar, header, 'COMMENT', 'Orignal header created by S. Shumko'
   fxaddpar, header, 'COMMENT', 'Header partially modified by K. Ahn'

   if calib then writefits, patho+'df'+dl+'dfc_'+ftime+'.fts', imgc, header  $
    else writefits, patho+'df'+dl+'df_'+ftime+'.fts', fix(rebin(img, NX/binx, NY/biny, nf)), header
   if dfcomp and platform eq 'win' then spawn,'D:\fitsio\fpack -r -C -D '+patho+'df\df_'+ftime+'.fts', /hide
   if dfcomp and platform eq 'X' then stop
   

        
        
      
        

   if comp and running_erase then file_delete, fout
   
  JUMP1:


  endfor


!p.multi=[0,1,1]

if calib then !null=temporary(imgc) else begin
  !null=temporary(img)
  !null=temporary(ss)
  endelse

endif

;_________________ALIGN & STOKES CALCULATION___________________

if (align) then begin

  
  if c2048 then begin
    exposure=15.4
    t_drift=0.;15.
    ph_drift=t_drift/exposure*22.5/2.
  endif else begin
    exposure=35.;temp exposure time
    t_drift=31.;33.;temp expected drift time
    ph_drift=t_drift/exposure*22.5/2.;maximum possible phase offset from the 1/4 & 3/4 portion of the chip due to ripple readout
  endelse

  Qsign=fltarr(nf)
  Usign=fltarr(nf)
  Vsign=fltarr(nf)

  for jj=0, nf-1 do begin
    if (jj mod 4 eq 0) or (jj mod 4 eq 3) then Qsign[jj]=1. else Qsign[jj]=-1.
    if (jj mod 4 eq 0) or (jj mod 4 eq 1) then Usign[jj]=1. else Usign[jj]=-1.
  endfor


  Qwt=!pi/1.6
  Uwt=!pi/1.6
  Vwt=!pi/1.6


  if c2048 then begin
    bint=binx
    binx=biny
    biny=bint
   
  endif
  
    Isig=intarr(NX/2/binx,NY/biny, nw); & Isig(*)=0
    Qsig=intarr(NX/2/binx,NY/biny, nw); & Qsig(*)=0
     Qsigt=intarr(NX/2/binx, NY/biny, nw); & Qsigt(*)=0
    Usig=intarr(NX/2/binx,NY/biny, nw); & Usig(*)=0
    Vsig=intarr(NX/2/binx,NY/biny, nw); & Vsig(*)=0
    Tsig=intarr(NX/2/binx,NY/biny, nw, 4)
  
  ;REBIN THE IMAGE TO 1/2
  window, 0, xs=NX/binx,  ys=NY/biny



  qoffsett=fltarr(ie-is+1)

 if date ge '20141111' then  posoffset=-2300. else posoffset=-3400.; was -3400 until 11/11/14
                          ;-3100. for 2015?
  ;MAIN LOOP____________________________________________


  index=filelst



  iindex=file_search(patho+'df'+dl+'df*.fts*')
  pindex=file_search(patho+'position'+dl+'frtimes_*.txt')


 

  for i=is, ie do begin

if i eq is then ta=systime(/seconds)


    
    if strmid(index[i], strlen(index[i])-2, 2) eq 'fz' then ftime=strmid(index[i], strlen(index[i])-23, 13) $
      else ftime=strmid(index[i], strlen(index[i])-20, 13)

    if calib then itest=file_test(patho+'final'+dl+'C_'+ftime+'.fts') $
      else itest=file_test(patho+'final'+dl+'I_'+ftime+'.fts')

    if itest and (i ne is) then begin
      print, ftime+' already processed. Skipping...'
      goto, JUMP2
    endif

    if calib then ftest=file_test(patho+'df'+dl+'dfc_'+ftime+'.fts') or file_test(patho+'df'+dl+'dfc_'+ftime+'.fts.fz') $
      else ftest=file_test(patho+'df'+dl+'df_'+ftime+'.fts') or file_test(patho+'df'+dl+'df_'+ftime+'.fts.fz')
    ptest=file_test(patho+'position'+dl+'frtimes_20'+ftime+'.txt')

    if ftest*ptest ne 1 then begin
      print, 'Error: no matching position file for FITS file, skipping '+ftime
      ;stop
      goto, JUMP2
    endif
    
    ;TEST FILES & READ IMG
    if strmid(iindex[i], strlen(iindex[i])-2, 2) eq 'fz' then begin
      fout=NIRIS_decomp(iindex[i], t_comp_path)
      if fout ne '' then begin
        acomp=1
      endif else begin
        print, 'Error: failed to find the decompressed file...'
        stop
      endelse
    endif else acomp=0

    if acomp then img=readfits(file_search(fout), header, /silent) else begin
      if calib then img=readfits(file_search(patho+'df'+dl+'dfc_'+ftime+'.fts'), header, /silent) $
      else img=readfits(file_search(patho+'df'+dl+'df_'+ftime+'.fts'), header, /silent)
    endelse
      
    if fxpar(header, 'NAXIS3') ne nf then begin
      print, 'Skipping '+ftime+' due to nf mismatch'
      goto, JUMP2
    endif
    
   
    ;rotate image 
    if c2048 and ~imrot then begin
      if binx ne biny then begin
        imgt=intarr(n_elements(img[0,*,0]), n_elements(img[*,0,0]), n_elements(img[0,0,*]))
        for j=0, nf-1 do imgt[*,*,j]=rotate(img[*,*,j], 1)
        img=imgt
        !null=temporary(imgt)
      endif else for j=0, nf-1 do img[*,*,j]=rotate(img[*,*,j], 1)
    endif

    ;add postion offset information
    fxaddpar, header, 'POSOFFSET', posoffset, 'Manual posision offset value in step'
    
    ;Rotate image b
    if ~calib then begin
      ;tEMP
      if file_test(patho+date+'_shifts.sav') eq 0 then load_shifts=0
      
      if i eq 0 and ~load_shifts then begin
        istd=10e10
          tal0=systime(/seconds)
          for k=-1., 1., 0.05 do begin
            for l=-40/binx,40/binx, 2/binx  do begin
              for m=-30/biny, 30/biny, 2/biny do begin
                imgt=shift(rot(img[*,*,0], k, /interp), l, 0)
                imgtb=rotate(shift(imgt[chipsz/2/binx:*,*], 0, m),5)
                if c2048 and ~ROIs then istdt=stddev(imgt[156*2/binx:400*2/binx,412*2/biny:656*2/biny]-imgtb[156*2/binx:400*2/binx,412*2/biny:656*2/biny])
                if c2048 and ROIs then istdt=stddev(imgt[156*2/binx:400*2/binx,156*2/biny:400*2/biny]-imgtb[156*2/binx:400*2/binx,156*2/biny:400*2/biny]) 
                if ~c2048 then istdt=stddev(imgt[15*2/binx:240*2/binx,120*2/biny:390*2/biny]-imgtb[15*2/binx:240*2/binx,120*2/biny:390*2/biny])
                 
                if istdt lt istd then begin
                  istd=istdt & bk=k & bl=l &bm=m
                  print, istd, k, l, m
                endif
            endfor
          endfor
          endfor
          
          shift_done=0
          repeat begin
                imgtf=ch_shift_sub(rot(img[*,*,0], bk, /interp), bl, 0)
                imgtf[chipsz/2/binx:*,*]=rotate(ch_shift_sub(imgtf[chipsz/2/binx:*,*], 0, bm),5)
            out_diff=bytscl(imgtf[0:chipsz/2/binx-1,*]-imgtf[chipsz/2/binx:chipsz/binx-1,*], -100, 100)
            tv, out_diff, 0
            xyouts, 0, 0, string(bk)+string(bl)+string(bm)
            write_png, patho+date+'_difference.png', out_diff
            print, bk, bl, bm
            read, bk, bl, bm, shift_done, prompt='bk, bl, bm to modify. When done, add 1 else 0: '
            endrep until shift_done eq 1
            
          save, bk, bl, bm, filename=patho+date+'_shifts.sav'
          
      endif else restore, filename=patho+date+'_shifts.sav'

      for j=0, nf-1 do begin
        img[*,*,j]=ch_shift_sub(rot(img[*,*,j], bk, /interp), bl, 0)
        img[chipsz/2/binx:*,*,j]=rotate(ch_shift_sub(img[chipsz/2/binx:*,*,j], 0, bm),5)
      endfor

    endif

    ;START POSITION DATA PROCESSING________________

    ;Get position data
    readcol, patho+'position'+dl+'frtimes_20'+ftime+'.txt', f='i,f', n, position, /sil

    ;Skip this loop if position file is corrupted
    if n_elements(where (position eq 0)) gt 110 then begin
      print, 'Postion file corrupted for '+ftime+'. Skipping...'
      goto, JUMP2
    endif

    position+=posoffset

    ;Correct for negative step values
    for j=0, nf-1 do begin
      if (position[j] lt 0) then position[j]+=25600
      if (position[j] gt 25600) then position[j]-=25600
    endfor

    ;Determine phase of modulation at zero step position
    all_angle=position/25600*360d
    med_phase=median(all_angle mod 22.5); 0<= med_phase < 22.5
    abs_array=abs(all_angle - med_phase)
    zindex_arr=where(abs_array eq min(abs_array))
    zindex=zindex_arr[0] mod 16

    phase=((360.-(zindex+1)*22.5) + med_phase) mod 360.;0<= phase  <360
    phaseV=((phase + 11.25) mod 22.5) - 11.25; -11.25 <= phaseV <11.25
    shiftV= fix((phase - phaseV)/22.5);0<= shiftV =< 16


    Vsign(*)=0.
    for jj=0, nf-1 do begin
      if (jj mod 8 ge 4) then Vsign[jj]=1. else Vsign[jj]=-1.
    endfor

    Vsign=shift(Vsign, 16-shiftV)
    Vsign[0:15]=Vsign[16:31]

    ;Calculate phase at all y positions
    if ~c2048 then begin 
    phase_ydir= transpose(transpose(replicate(1., chipsz/2/binx))## $
        (phase - (abs( findgen(chipsz/biny) - chipsz/2/biny)-chipsz/4/biny )/(chipsz/2/biny) *t_drift/exposure*22.5) )
    phase_ydirV=transpose(transpose(replicate(1., chipsz/2/binx))## $
        (phaseV -(abs( findgen(chipsz/biny) - chipsz/2/biny)-chipsz/4/biny )/(chipsz/2/biny) *t_drift/exposure*22.5) )
    endif
    ;END OF POSITION DATA PROCESSING___________________

    if i eq is then begin
    tc=systime(/seconds)
    print, 'Start image align at '+string(tc-ta)
    endif

    ;START IMAGE ALIGNMENT_____________________________

    ;Compensate for beam wobble for the whole burst
    if (i eq is and ~calib) then begin
      if (ROI0) then begin
      ss0=xdefroi(img[*,*,1]);was 0
      se0=n_elements(ss0)-1
      xs0=ss0[0] mod (chipsz/binx) & xe0=ss0[se0] mod (chipsz/binx)
      ys0=ss0[0]/(chipsz/biny) & ye0=ss0[se0]/(chipsz/biny)
      endif else begin
       if c2048 and ~ROIs then begin
         ys0=824/biny & ye0=1224/biny;xs was 529
         xs0=312/binx & xe0=800/binx
       endif else if c2048 and ROIs then begin
         ys0=312/biny & ye0=800/biny;xs was 529
         xs0=312/binx & xe0=800/binx       
        endif else begin
         ys0=229/biny & ye0=691/biny;xs was 529
         xs0=100/binx & xe0=411/binx
       endelse
      endelse
    endif
    
    offset0arr=fltarr(2,nf)
    for j=0, nf-1 do begin

      if ~calib then begin
        offset0=ch_alignoffset(img[xs0:xe0,ys0:ye0,j], img[xs0:xe0,ys0:ye0,0], cor)
	      if j ge 8*18 and abs(offset0[0]^2+offset0[1]^2) gt 100 then offset0=offset0arr[*,j-8*18]
	      img[*,*,j]=ch_shift_sub(img[*,*,j], -1.*offset0[0], -1.*offset0[1])
        offset0arr[*,j]=offset0

        if j eq 0 then  tv, bytscl(img[*,*,j],  3000, 6000 )
      endif
    endfor

    ;left half - right half = modulation in dual beam
    if c2048 and calib then begin 
    a=img[0:15,*,*];left  half
    b=img[16:31,*,*]; right half
    endif else begin
    a=img[0:chipsz/2/binx-1,*,*];left  half
    b=img[chipsz/2/binx:chipsz/binx-1,*,*]; right half
    endelse
    !null=temporary(img)
    ref=a[*,*,0]
    
    if (i eq is and ~calib) then begin
      if (ROI1) then begin
        ss=xdefroi(a[*,*,0])
        se=n_elements(ss)-1
        xs=ss[0] mod (chipsz/2/binx) & xe=ss[se] mod (chipsz/2/binx)
        ys=ss[0]/(chipsz/biny) & ye=ss[se]/(chipsz/biny)
      endif else begin
       if c2048 and ROIs then begin
         ys=250/biny & ye=800/biny;xs was 501
         xs=250/binx & xe=800/binx
       endif else if c2048 and ~ROIs then begin
         ys=762/biny & ye=1312/biny;xs was 501
         xs=250/binx & xe=800/binx
       endif else begin
              ys=229/biny & ye=691/biny;xs was 529
      xs=100/binx & xe=411/binx
       endelse
      endelse
    endif
    
    if ~calib then begin
      offsetbarr=fltarr(2,nf)
      for j=0, nf-1 do begin
        if j eq 0 then offsetb=ch_alignoffset(b[xs:xe,ys:ye,j], ref[xs:xe,ys:ye])
        if j ge 8*18 and abs(offsetb[0]^2+offsetb[1]^2) gt 100 then offsetb=offsetbarr[*,j-8*18]

        b[*,*,j]=ch_shift_sub(b[*,*,j], -1.*offsetb[0], -1.*offsetb[1])

        offsetbarr[*,j]=offsetb
        if j eq 0 then tv, bytscl(a[*,*,j]-b[*,*,j], -100, 100), 1

      endfor
    endif

    if i eq is then begin
    td=systime(/seconds)
    print, 'Start destrech at '+string(td-ta)
    endif

    ;DESTRETCH____________________________________________
    if (align_dest and ~calib) then begin; destrech only when used as I data
      ;if binx ne 2 or biny ne 2 then stop
      kernel=bytarr(50/binx, 50/biny)

      ret=fltarr(2,2)
      ret2=fltarr(2,2)
      if c2048 and ~ROIs then ret_arr=fltarr(2,20,40,16)
      if c2048 and ROIs then ret_arr=fltarr(2,20,20,16)  
      if ~c2048 then ret_arr=fltarr(2,9,20,16)  

      for j=0, 15 do begin
        temp_arr=reg(b[*,*,j], a[*,*,j], kernel, ret, deb=3)
        ret_arr[*,*,*,j]=ret[*,*,*,0]
      endfor



      ret_all=median(ret_arr, dimension=4)

      for j=0, nf-1 do b[*,*,j]=doreg(b[*,*,j], ret[*,*,*,1], ret_all)
  
  
      if i eq is then begin
        tda=systime(/seconds)
        print, 'Done correcting b '+string(tda-ta)
      endif

      for j=0, nf-1 do begin
        if j mod nfpw le 15 then begin
          ;a[*,*,j]=reg(a[*,*,j], a[*,*,0], kernel, deb=0);temp
          ;b[*,*,j]=reg(b[*,*,j], a[*,*,0], kernel, deb=0);temp
          a[*,*,j]=reg(a[*,*,j], a[*,*,0], kernel, ret2, deb=0)
          b[*,*,j]=doreg(b[*,*,j], ret2[*,*,*,1], ret2[*,*,*,0])
        endif
      endfor


    endif



    ;END OF IMAGE ALIGNMENT___________________________________



    if i eq is then begin
    te=systime(/seconds)
    print, 'Start Stokes cal. at '+string(te-ta)
    endif

    ;START STOKES SIGNAL EXTRACT_______________________________

    ;Initialize all the signal arrays

    Isig(*)=0 & Qsig(*)=0 & Qsigt(*)=0 & Usig(*)=0 & Vsig(*)=0


    ;determine constant offset in a-b caused by Q signal
   ; Qoffset=intarr(512/binx, 1024/biny, nw)
   ; for j=0, nf-1, nfpw do Qoffset[*,*,j/nfpw]=total(a[*,*,j:j+15]-b[*,*,j:j+15], 3)/16

    for j=0, nf-1 do begin
      if(j mod nfpw le 15) then begin
          Isig[*,*,j/nfpw] += (a[*,*,j]+b[*,*,j])/16
          Qsig[*,*,j/nfpw] += (a[*,*,j]-b[*,*,j])*Qsign[j]*Qwt/16
          Usig[*,*,j/nfpw] += (a[*,*,j]-b[*,*,j])*Usign[j]*Uwt/16
          Vsig[*,*,j/nfpw] += (a[*,*,j]-b[*,*,j])*Vsign[j]*Vwt/16
      endif
    endfor

    !null=temporary(a)
    !null=temporary(b)

;Compensate for phase drift caused by modulator
    Qsigt=Qsig
    for j=0, nf-1, nfpw do begin
      if ~c2048 then begin
        Qsig[*,*,j/nfpw]=cos(4.*phase_ydir/!radeg)*Qsigt[*,*,j/nfpw]-sin(4.*phase_ydir/!radeg)*Usig[*,*,j/nfpw]
        Usig[*,*,j/nfpw]=sin(4.*phase_ydir/!radeg)*Qsigt[*,*,j/nfpw]+cos(4.*phase_ydir/!radeg)*Usig[*,*,j/nfpw]
        Vsig[*,*,j/nfpw]/=cos(2.*phase_ydirV/!radeg)
      endif else begin;TEMP. phase & phaseV should be dependent on rows
        Qsig[*,*,j/nfpw]=cos(4.*phase/!radeg)*Qsigt[*,*,j/nfpw]-sin(4.*phase/!radeg)*Usig[*,*,j/nfpw]
        Usig[*,*,j/nfpw]=sin(4.*phase/!radeg)*Qsigt[*,*,j/nfpw]+cos(4.*phase/!radeg)*Usig[*,*,j/nfpw]
        Vsig[*,*,j/nfpw]/=cos(2.*phaseV/!radeg)  
      endelse
    endfor


    file_mkdir, patho+'final'
    Tsig[*,*,*,0]=Isig
    Tsig[*,*,*,1]=Qsig;)/float(Isig)
    Tsig[*,*,*,2]=Usig;)/float(Isig)
    Tsig[*,*,*,3]=Vsig;)/float(Isig)

;    ;Remove uneven Q background for selected dates
;    if rm_bk and nw eq 105 then begin
;      if i eq is then begin print, 'Uneven background in Q is being removed...'
;      qbgr=median(Tsig[*,*,45:64,1], dimension=3)
;      bg_prof=median(median(Tsig[128:241,71:193,*,1], dimension=1), dimension=1)
;      bg_xsam=[findgen(10), findgen(20)+45, findgen(10)+95]
;      bg_ysam=fltarr(40)
;
;      bg_ysam[0:9]=bg_prof[0:9]
;      bg_ysam[10:29]=bg_prof[45:64]
;      bg_ysam[30:39]=bg_prof[95:104]
;
;      bg_xint=findgen(105)
;
;      bg_pint=interpol(bg_ysam, bg_xsam, bg_xint)
;      bg_pint/=median(bg_pint[45:64])
;
;      plot, bg_xint, bg_pint
;      save, qbgr, bg_pint, filename=patho+date+'_curve.sav'
;      endif else restore, filename=patho+date+'_curve.sav'
;      for wv=0, 104 do Tsig[*,*,wv,1]-=(qbgr*bg_pint[wv])
;
;    endif

    curt=systime()
    fxaddpar, header, 'HISTORY', 'Created Stokes map at '+curt
    fxaddpar, header, 'HISTORY', 'Changed file type to int'

    if calib then writefits, patho+'final'+dl+'C_'+ftime+'.fts', Tsig, header $
      else writefits, patho+'final'+dl+'I_'+ftime+'.fts', Tsig, header
    
   mt=median(Isig) & stdt=2.*stddev(Isig)
    
    if calib then thi=3*mt else thi=0.03*mt
    tlo=-1.*thi

    

    if calib then begin 
      xu=NX/binx/2
      yu=NY/binx
    endif else begin
      xu=NX/binx/2 
       yu=NY/biny
       endelse
    
    if i eq is then window, 1, xs=xu*4, ys=yu*2
    wset, 1

    if calib then begin
      if c2048 then begin
        xd=51 & yd=512
      endif else begin
        xd=51 & yd=512
      endelse
    
      for r=0, 4 do begin
        tv, congrid(bytscl(Isig[*,*,r], mt-stdt, mt+stdt),xd, yd), 0*5+r
        tv, congrid(bytscl(Tsig[*,*,r,1], tlo, thi),xd, yd), 1*5+r
        tv, congrid(bytscl(Tsig[*,*,r,2], tlo, thi),xd, yd), 2*5+r
        tv, congrid(bytscl(Tsig[*,*,r,3], tlo, thi),xd, yd), 3*5+r
      endfor
    endif else begin
        tv, rebin(bytscl(Isig[*,*,round(nw/3.6)], mt-stdt, mt+stdt),xu, yu), 0
        tv, rebin(bytscl(Tsig[*,*,round(nw/3.6),1], tlo, thi),xu, yu), 1
        tv, rebin(bytscl(Tsig[*,*,round(nw/3.6),2], tlo, thi),xu, yu), 2
        tv, rebin(bytscl(Tsig[*,*,round(nw/3.6),3], tlo, thi),xu, yu), 3
        tv, rebin(bytscl(Isig[*,*,round(nw/4.8)], mt-stdt, mt+stdt),xu, yu), 4
        tv, rebin(bytscl(Tsig[*,*,round(nw/4.8),1], tlo, thi),xu, yu), 5
        tv, rebin(bytscl(Tsig[*,*,round(nw/4.8),2], tlo, thi),xu, yu), 6
        tv, rebin(bytscl(Tsig[*,*,round(nw/4.8),3], tlo, thi),xu, yu), 7
    endelse

    png_canvas=fltarr(xu*4,yu*2)
    png_canvas[0:xu-1,yu:2*yu-1]=rebin(bytscl(Isig[*,*,round(nw/3.6)], mt-stdt, mt+stdt),xu, yu)
    png_canvas[xu:2*xu-1,yu:2*yu-1]=rebin(bytscl(Tsig[*,*,round(nw/3.6),1], tlo, thi),xu, yu)
    png_canvas[2*xu:3*xu-1,yu:2*yu-1]=rebin(bytscl(Tsig[*,*,round(nw/3.6),2], tlo, thi),xu, yu)
    png_canvas[3*xu:4*xu-1,yu:2*yu-1]=rebin(bytscl(Tsig[*,*,round(nw/3.6),3], tlo, thi),xu, yu)
    png_canvas[0:xu-1,0:yu-1]=rebin(bytscl(Isig[*,*,round(nw/4.8)], mt-stdt, mt+stdt),xu, yu)
    png_canvas[xu:2*xu-1,0:yu-1]=rebin(bytscl(Tsig[*,*,round(nw/4.8),1], tlo, thi),xu, yu)
    png_canvas[2*xu:3*xu-1,0:yu-1]=rebin(bytscl(Tsig[*,*,round(nw/4.8),2], tlo, thi),xu, yu)
    png_canvas[3*xu:4*xu-1,0:yu-1]=rebin(bytscl(Tsig[*,*,round(nw/4.8),3], tlo, thi),xu, yu)

    if calib then write_png, patho+'final'+dl+'C_'+ftime+'.png', png_canvas $
      else write_png, patho+'final'+dl+'I_'+ftime+'.png', png_canvas

    if i eq is then begin
    tf=systime(/seconds)
    print, 'A loop done at '+string(tf-ta)
    endif
    
    wset, 0
    
    if acomp and running_erase then file_delete, fout
    
    
    JUMP2:
  endfor

  !p.multi=[0,4,4]

endif


return, 1
!p.multi=[0,1,1]


end
