function NIRIS_raw2Stokes_pc, date, calib, makednf, flatndark, align, surfix, prefixi, prefixo
;+
; IRIM_raw2Stokes.pro
;
; :Author: KWANGSU
; This program produces 4 Stokes signals from dual beam observation.
;- Major modification: 
;  / / : Changed orientation of dual beam from horizontal split to vertical split. Ver. 1.5
;  07/10/2014: Changed orientation of the polarization beam splitter so that dual beam changes signs. : Ver. 1.6
;  07/16/2014: Changed code in compensation of phase.:Ver. 2.0
;
; The output data type:
; img[1024/binx, 512/biny, 13, 4]
; the 4th elements are
; 0: intensity profile (not normalized to the continuum)
; 1: Stokes Q (phase shift corrected, ripple readout not corrected)
; 2: Stokes U
; 3: Stokes V


;makednf: Create dark and flat
;flatndark: Perform dark subtraction and flat fielding from raw images
;align: Align frames and get results

; manf: if flat files should be chosen manually since flat data are in a form of image data
;        in this case, users must copy the flat files into the 'calib' folder of processing disk
;        to see what files should be copied, please see the log files in the raw data.


;PARAMETERS
set_plot, 'win'

patho=prefixo+date+'\'+surfix
file_mkdir, patho
;copy position 0: no copy, 1: copy w/ovewrite, 2: copy while taking data
copy_position=1

 pathi=prefixi+date+'\'+surfix
;------------------------------------------------
if makednf then begin

  year=strmid(date,0,4)
  month=strmid(date,4,2)
  day=strmid(date,6,2)
  case month of 
  '01':monc='jan'
  '02':monc='feb'
  '03':monc='mar'
  '04':monc='apr'
  '05':monc='may'
  '06':monc='jun'
  '07':monc='jul'
  '08':monc='aug'
  '09':monc='sep'
  '10':monc='oct'
  '11':monc='nov'
  '12':monc='dec'

  endcase

 
  testindex=file_search(pathi+'*.*')

  if testindex[0] eq '' then begin
    print, 'No file found.'
    stop
  endif

  if calib eq 1 then filelst=file_search(pathi+'calib\*_c_*.fts') else filelst=file_search(pathi+'*_i_*.fts')
  help, filelst
  
      ;SIDE JOB: COPY POSITION files ____________
if copy_position ne 0 then begin
   print, 'Copying position files...'

  if calib eq 1 then pindex=file_search(pathi+'calib\frtimes_*.txt') $
    else pindex=file_search(pathi+'*.txt')

  if n_elements(pindex) ge 1 then begin
    file_mkdir, patho+'position'
    if copy_position eq 1 then file_copy, pindex, patho+'position', /overwrite $
      else file_copy, pindex[0:nm1(pindex)-1], patho+'position', /overwrite
  print, 'Done cpy position'
    
  endif
  ;END COPY POSITION

endif
  
  ;check wavelength number
  nf_array=intarr(n_elements(filelst))

  print, 'Checking median number of frames...'
  for i=0, nm1(filelst) do begin
    headers=headfits(filelst[i], /sil)
    nf_array[i]=fxpar(headers, 'NAXIS3')
  endfor

  nf=fix(median(nf_array))
  nfpw=18
  nw=nf/nfpw
  if nf mod nfpw ne 0 then stop
  print, 'nw=', nw

  nf=nfpw*nw

  save, nf, nfpw, nw, filelst, filename=patho+date+'_params.sav'
  stop

endif
;-----------------------------------------------------
if align then begin
;TO DETERMINE A REGEON OF INTEREST FOR ALIGNMENT.
;OTHERWISE, THE ALIGNMENT WILL BE DONE WITH PRESET ROIs.
ROI0=0; Alignment of frames in a burst
ROI1=0; Alignment of dual beams
 ;calib overrides ROI0 and ROI1
 ;with calib, threshold to mask bad pixels will not work

align_dest=1
;set rotation degree for align
drot=2.2
if date ge '20140601' then drot=3.5
if date ge '20140701' then drot=2.8
if date ge '20140801' then drot=-2.65

print, 'Image derotation by '+string(drot)+' degree was applied.'

if date ge '20140601' and date lt '20140610' then rm_bk=1 else rm_bk=0

endif
;-------------------------------------------------------

;__________________________________________
if makednf ne 1 then restore, filename=patho+date+'_params.sav'
is=0;Starting burst (0 is the first)
ie=nm1(filelst);Ending burst
;__________________________________________


if calib eq 1 then begin 
  binx=512
  biny=2
endif else begin
  binx=2
  biny=2
endelse


  

;_____________CREATE DARK AND FLAT_____________________________________

if( makednf ) then begin



  flatindex=file_search(pathi+'calib\bbso_irim*_f_*.fts')
  darkindex=file_search(pathi+'calib\bbso_irim*_d_*.fts')




  ;Make dark (dark should be one file)
  darkraw=readfits(darkindex[0], dh, /sil)
  dark=fix(median(darkraw, dimension=3))
  file_mkdir, patho+'calib'
  writefits, patho+'calib\dark.fts', dark, dh

  ;Make flat
  flatarray=intarr(1024,1024,nw)
  itemp=intarr(1024,1024,nf)
  for i=0, n_elements(flatindex)-1 do begin
    if (i eq 0) then itemp+=(readfits(flatindex[i], fh, /silent)/n_elements(flatindex)) $ 
      else itemp+=(readfits(flatindex[i],/silent)/n_elements(flatindex))
  endfor

  for i=0, nw-1 do flatarray[*,*,i]=total(itemp[*,*,i*nfpw:i*nfpw+15], 3)/16

  writefits, patho+'calib\flat.fts', flatarray, fh
  !null=temporary(itemp)
  !null=temporary(darkraw)
  !null=temporary(flatarray)
  !null=temporary(dark)
  
  print, 'Flat & dark made...'
endif

;_____________________________________________________________


;_____________PROCESS DARK & FLAT_____________________________

if (flatndark) then begin


  file_mkdir, patho+'df'
  index=filelst

  dark=readfits(patho+'calib\dark.fts', /silent)
  flat=readfits(patho+'calib\flat.fts', /silent)
  
  fdfac=intarr(nw)

  fd=intarr(1024,1024,nw)
  for k=0, nw-1 do begin
    fd[*,*,k]=flat[*,*,k]-dark
    fdfac[k]=median(fd[*,*,k])
  endfor

  ;Look for bad pixels switch to the median value

  if calib ne 1 then begin
    ss=where(fd le 1d3)
    fd[ss]=median(fd)
  endif

  file_mkdir, patho+'df'
  set_plot, 'win'
  window, 0, xs=1024, ys=1024
  !p.multi=[0,1,3]



  for i=is, ie do begin
    ;MODIFIED
    ftime=strmid(index[i], strlen(index[i])-20, 13)
    ftest=file_test(patho+'df\df_'+ftime+'.fts')
    if ftest then begin
      print, ftime+' already exists. Skipping...'
      goto, JUMP1
    endif
    
    img=readfits(index[i], header, /silent)
    if fxpar(header, 'NAXIS3') ne nf then begin
      print, 'frame number does not match for'+string(i)+'.'
      goto, JUMP1
    endif


    ;modify header for pixel scale
    pixscale=fxpar(header, 'CDELT1')
    pixscalex=pixscale*binx
    pixscaley=pixscale*biny
   
    fxaddpar, header, 'CDELT1', pixscalex
    fxaddpar, header, 'CDELT2', pixscaley

    for j=0, nf-1 do img[*,*,j]= fix(float(img[*,*,j]-dark)/fd[*,*,j/nfpw]*fdfac[j/nfpw])

    if calib ne 1 then begin
      ss=where(img[*,*,0] le 1d3)
      for j=0, nf-1 do begin
        imgj=img[*,*,j]
        imgj[ss]=imgj[ss+5]
        img[*,*,j]=imgj
      endfor
    endif

    ;masking bad columns
    img[253,*,*]=img[252,*,*] & img[254,*,*]=img[252,*,*] & img[255,*,*]=img[256,*,*]
    img[509,*,*]=img[508,*,*] & img[510,*,*]=img[508,*,*] & img[511,*,*]=img[512,*,*]
    img[765,*,*]=img[764,*,*] & img[766,*,*]=img[764,*,*] & img[767,*,*]=img[768,*,*]
    img[1021,*,*]=img[1020,*,*] & img[1022,*,*]=img[1020,*,*] & img[1023,*,*]=img[1020,*,*]

    imgh=img[*,*,0]
    hotmed=median(imgh)
    hotstd=stddev(imgh)
    kernel=[-0.5, 0.5]
    hotpix=where(convol(float(imgh), kernel) gt 100)
    hotx=hotpix mod n_elements(imgh[*,0])
    hoty=hotpix/n_elements(imgh[0,*])

    

    for j=0, nf-1 do begin
      imgj=img[*,*,j]
      for ti=0, 1 do begin
        for tj=-1, 1 do begin
          imgj[hotx+ti, hoty+tj]=(imgj[hotx+5, hoty]+imgj[hotx-5, hoty]+imgj[hotx, hoty+5]+imgj[hotx, hoty-5])/4
        endfor
      endfor

      if j eq 1 then begin
	tv, bytscl(imgj, hotmed-hotstd, hotmed+hotstd)
 	if calib ne 1 then write_png, patho+'df\df_'+ftime+'.png', bytscl(imgj, hotmed-hotstd, hotmed+hotstd)
      endif
      img[*,*,j]=imgj
    endfor


    if calib eq 1 then begin
      imgc=fltarr(1024/binx,1024/biny, nf)

      for j=0, nf-1 do begin
        imgc[0,*,j]=rebin(median(img[30:481,*,j], dimension=1), 512)
        imgc[1,*,j]=rebin(median(img[541:993,*,j], dimension=1), 512)
      endfor


      writefits, patho+'df\dfc_'+ftime+'.fts', imgc, header
    endif else writefits, patho+'df\df_'+ftime+'.fts', fix(rebin(img, 1024/binx, 1024/biny, nf)), header

  JUMP1:

  
  endfor


!p.multi=[0,1,1]

if calib then !null=temporary(imgc) else begin
  !null=temporary(img)
  !null=temporary(ss)
  endelse

endif

;_________________ALIGN & STOKES CALCULATION___________________

if (align) then begin



  exposure=35.;temp exposure time
  t_drift=31.;33.;temp expected drift time
  ph_drift=t_drift/exposure*22.5/2.;maximum possible phase offset from the 1/4 & 3/4 portion of the chip due to ripple readout

  Qsign=fltarr(nf)
  Usign=fltarr(nf)
  Vsign=fltarr(nf)

  for jj=0, nf-1 do begin
    if (jj mod 4 eq 0) or (jj mod 4 eq 3) then Qsign[jj]=1. else Qsign[jj]=-1.
    if (jj mod 4 eq 0) or (jj mod 4 eq 1) then Usign[jj]=1. else Usign[jj]=-1.
  endfor


  Qwt=!pi/1.6
  Uwt=!pi/1.6
  Vwt=!pi/1.6

;Was intarr
  Isig=intarr(512/binx,1024/biny, nw); & Isig(*)=0
  Qsig=intarr(512/binx,1024/biny, nw); & Qsig(*)=0
   Qsigt=intarr(512/binx, 1024/biny, nw); & Qsigt(*)=0
  Usig=intarr(512/binx,1024/biny, nw); & Usig(*)=0
  Vsig=intarr(512/binx,1024/biny, nw); & Vsig(*)=0
  Tsig=fltarr(512/binx,1024/biny, nw, 4)

  ;REBIN THE IMAGE TO 1/2
  window, 0, xs=512,  ys=512



  qoffsett=fltarr(ie-is+1)

  posoffset=-3400.

  ;MAIN LOOP____________________________________________


  index=filelst

 

  iindex=file_search(patho+'df\df*.fts')
  pindex=file_search(patho+'position\frtimes_*.txt')


  for i=is, ie do begin

if i eq is then ta=systime(/seconds)

    ;MODIFIED
    ftime=strmid(index[i], strlen(index[i])-20, 13)

    if calib then itest=file_test(patho+'final\C_'+ftime+'.fts') $
      else itest=file_test(patho+'final\I_'+ftime+'.fts')
    
    if itest and (i ne is) then begin
      print, ftime+' already processed. Skipping...'
      goto, JUMP2
    endif

    if calib then ftest=file_test(patho+'df\dfc_'+ftime+'.fts') $
      else ftest=file_test(patho+'df\df_'+ftime+'.fts')
    ptest=file_test(patho+'position\frtimes_20'+ftime+'.txt')

    if ftest*ptest ne 1 then begin
      print, 'Error: no matching position file for FITS file, skipping '+ftime
      goto, JUMP2
    endif

    if calib eq 1 then begin
      img=readfits(file_search(patho+'df\dfc_'+ftime+'.fts'), header, /silent)
    endif else img=readfits(file_search(patho+'df\df_'+ftime+'.fts'), header, /silent)
    
    if fxpar(header, 'NAXIS3') ne nf then begin
      print, 'Skipping '+ftime+' due to nf mismatch'
      goto, JUMP2
    endif
 
    ;Rotate image b
    if calib eq 0 then for j=0, nf-1 do img[256:511,*,j]=rot(rotate(img[256:511,*,j], 5), drot, /interp)
   


    ;START POSITION DATA PROCESSING________________

    ;Get position data
    readcol, patho+'position\frtimes_20'+ftime+'.txt', f='i,f', n, position, /sil

    ;Skip this loop if position file is corrupted
    if n_elements(where (position eq 0)) gt 110 then begin
      print, 'Postion file corrupted for '+ftime+'. Skipping...'
      goto, JUMP2
    endif

    position+=posoffset

    ;Correct for negative step values
    for j=0, nf-1 do begin
      if (position[j] lt 0) then position[j]+=25600
      if (position[j] gt 25600) then position[j]-=25600
    endfor
	 
    ;Determine phase of modulation at zero step position
    all_angle=position/25600*360d
    med_phase=median(all_angle mod 22.5); 0<= med_phase < 22.5
    abs_array=abs(all_angle - med_phase)
    zindex_arr=where(abs_array eq min(abs_array))
    zindex=zindex_arr[0] mod 16

    phase=((360.-(zindex+1)*22.5) + med_phase) mod 360.;0<= phase  <360
    phaseV=((phase + 11.25) mod 22.5) - 11.25; -11.25 <= phaseV <11.25
    shiftV= fix((phase - phaseV)/22.5);0<= shiftV =< 16


  Vsign(*)=0.
  for jj=0, nf-1 do begin
    if (jj mod 8 ge 4) then Vsign[jj]=1. else Vsign[jj]=-1.
  endfor

    Vsign=shift(Vsign, 16-shiftV) 
    Vsign[0:15]=Vsign[16:31]

    ;Calculate phase at all y positions
    phase_ydir= transpose(transpose(replicate(1., 512/binx))## $
        (phase - (abs( findgen(1024/biny) - 512/biny)-256/biny )/(512/biny) *t_drift/exposure*22.5) )
    phase_ydirV=transpose(transpose(replicate(1., 512/binx))## $
        (phaseV -(abs( findgen(1024/biny) - 512/biny)-256/biny )/(512/biny) *t_drift/exposure*22.5) )

    ;END OF POSITION DATA PROCESSING___________________ 
	  
if i eq is then begin
tc=systime(/seconds)
print, 'Start image align at '+string(tc-ta)
endif

    ;START IMAGE ALIGNMENT_____________________________

    ;Compensate for beam wobble for the whole burst
    if (i eq is and calib ne 1) then begin
      if (ROI0) then begin
      ss0=xdefroi(img[*,*,1]);was 0
      se0=n_elements(ss0)-1
      xs0=ss0[0] mod (1024/binx) & xe0=ss0[se0] mod (1024/binx)
      ys0=ss0[0]/(1024/biny) & ye0=ss0[se0]/(1024/biny)
      endif else begin
      ys0=229/biny & ye0=691/biny;xs was 529
      xs0=100/binx & xe0=411/binx
      endelse
    endif


    for j=0, nf-1 do begin
     
      if calib ne 1 then begin
        offset0=ch_alignoffset(img[xs0:xe0,ys0:ye0,j], img[xs0:xe0,ys0:ye0,0], cor)
	img[*,*,j]=ch_shift_sub(img[*,*,j], -1.*offset0[0], -1.*offset0[1])

         if j eq 0 then  tv, bytscl(img[*,*,j],  3000, 6000 )
      endif

    endfor

    ;left half - right half = modulation in dual beam
    a=img[0:511/binx,*,*];left  half
    b=img[512/binx:1023/binx,*,*]; right half
    !null=temporary(img)
    ref=a[*,*,0]

    if (i eq is and calib ne 1) then begin
      if (ROI1) then begin
        ss=xdefroi(a[*,*,0])
        se=n_elements(ss)-1
        xs=ss[0] mod (512/binx) & xe=ss[se] mod (512/binx)
        ys=ss[0]/(1024/biny) & ye=ss[se]/(1024/biny)
      endif else begin
       ys=201/biny & ye=694/biny;xs was 501
       xs=80/binx & xe=317/binx
      endelse
    endif

    if calib ne 1 then begin

      for j=0, nf-1 do begin
        if j eq 0 then offsetb=ch_alignoffset(b[xs:xe,ys:ye,j], ref[xs:xe,ys:ye])
        
        b[*,*,j]=ch_shift_sub(b[*,*,j], -1.*offsetb[0], -1.*offsetb[1])
        if j eq 0 then tv, bytscl(a[*,*,j]-b[*,*,j], -100, 100), 1
 
      endfor
    endif

if i eq is then begin
td=systime(/seconds)
print, 'Start destrech at '+string(td-ta)
endif

    ;DESTRETCH____________________________________________
    if (align_dest and calib eq 0) then begin; destrech only when used as I data
      if binx ne 2 or biny ne 2 then stop
      kernel=bytarr(50/binx, 50/biny)

      ret=fltarr(2,2)
      ret2=fltarr(2,2)
      ret_arr=fltarr(2,9,20,16)

      ;rett=fltarr(2,2)
      ;ret_arrt=fltarr(2,9,20,105)

      for j=0, 15 do begin
        temp_arr=reg(b[*,*,j], a[*,*,j], kernel, ret, deb=3)
        ret_arr[*,*,*,j]=ret[*,*,*,0]
      endfor
      
;      for j=0, nf-1, 18 do begin
;	temp_arr2=reg(b[*,*,j], a[*,*,j], kernel, rett, deb=0)
;  	ret_arrt[*,*,*,j/18]=rett[*,*,*,0]
;      endfor

      ret_all=median(ret_arr, dimension=4)
;      ret_allt=median(ret_arrt, dimension=4)

ttt=systime(/seconds)

      for j=0, nf-1 do begin
	b[*,*,j]=doreg(b[*,*,j], ret[*,*,*,1], ret_all)
;	temp=doreg(b[*,*,j], rett[*,*,*,1], ret_allt)
      endfor

tttt=systime(/seconds)
;	b1=reg(b, a[*,*,0], kernel, deb=0)
;ttttt=systime(/seconds)

print, tttt-ttt

if i eq is then begin 
tda=systime(/seconds)
print, 'Done correcting b '+string(tda-ta)  
endif
      
      for j=0, nf-1 do begin
        if j mod nfpw le 15 then begin
	  a[*,*,j]=reg(a[*,*,j], a[*,*,0], kernel, ret2, deb=0)
          b[*,*,j]=doreg(b[*,*,j], ret2[*,*,*,1], ret2[*,*,*,0])
          ;b[*,*,j]=reg(b[*,*,j], a[*,*,j], kernel, deb=0)  
        endif
      endfor

if i eq is then begin
tdb=systime(/seconds)
print, 'Done correcting among frms '+string(tdb-ta)
endif
    endif

     

    ;END OF IMAGE ALIGNMENT___________________________________



if i eq is then begin
te=systime(/seconds)
print, 'Start Stokes cal. at '+string(te-ta)
endif

    ;START STOKES SIGNAL EXTRACT_______________________________	

    ;Initialize all the signal arrays

    Isig(*)=0 & Qsig(*)=0 & Qsigt(*)=0 & Usig(*)=0 & Vsig(*)=0 


    ;determine constant offset in a-b caused by Q signal
   ; Qoffset=intarr(512/binx, 1024/biny, nw)
   ; for j=0, nf-1, nfpw do Qoffset[*,*,j/nfpw]=total(a[*,*,j:j+15]-b[*,*,j:j+15], 3)/16

    for j=0, nf-1 do begin
      if(j mod nfpw le 15) then begin
          Isig[*,*,j/nfpw] += (a[*,*,j]+b[*,*,j])/16
          Qsig[*,*,j/nfpw] += (a[*,*,j]-b[*,*,j])*Qsign[j]*Qwt/16
          Usig[*,*,j/nfpw] += (a[*,*,j]-b[*,*,j])*Usign[j]*Uwt/16
          Vsig[*,*,j/nfpw] += (a[*,*,j]-b[*,*,j])*Vsign[j]*Vwt/16
      endif
    endfor

;Compensate for phase drift caused by modulator
;TEMP
    Qsigt=Qsig
    for j=0, nf-1, nfpw do begin
      Qsig[*,*,j/nfpw]=cos(4.*phase_ydir/!radeg)*Qsigt[*,*,j/nfpw]-sin(4.*phase_ydir/!radeg)*Usig[*,*,j/nfpw]
      Usig[*,*,j/nfpw]=sin(4.*phase_ydir/!radeg)*Qsigt[*,*,j/nfpw]+cos(4.*phase_ydir/!radeg)*Usig[*,*,j/nfpw]
      Vsig[*,*,j/nfpw]/=cos(2.*phase_ydirV/!radeg)
    endfor


    file_mkdir, patho+'final'
    Tsig[*,*,*,0]=float(Isig)
    Tsig[*,*,*,1]=float(Qsig)/float(Isig)
    Tsig[*,*,*,2]=float(Usig)/float(Isig)
    Tsig[*,*,*,3]=float(Vsig)/float(Isig)

    fxaddpar, header, "COMMENT", "Ver. 2.5"	
    fxaddpar, header, "COMMENT", "improved destretch 08/08/2014"

    ;Remove uneven Q background for selected dates
    if rm_bk and nw eq 105 then begin
      if i eq is then print, 'Uneven background in Q is being removed...'
      qbgr=median(Tsig[*,*,45:64,1], dimension=3)
      bg_prof=median(median(Tsig[128:241,71:193,*,1], dimension=1), dimension=1)
      bg_xsam=[findgen(10), findgen(20)+45, findgen(10)+95]
      bg_ysam=fltarr(40)
  
      bg_ysam[0:9]=bg_prof[0:9]
      bg_ysam[10:29]=bg_prof[45:64]
      bg_ysam[30:39]=bg_prof[95:104]

      bg_xint=findgen(105)

      bg_pint=interpol(bg_ysam, bg_xsam, bg_xint)
      bg_pint/=median(bg_pint[45:64])

      plot, bg_xint, bg_pint
      for wv=0, 104 do Tsig[*,*,wv,1]-=(qbgr*bg_pint[wv])
      

    endif

    ftime=strmid(filelst[i], strlen(filelst[i])-20, 13)

    if calib eq 0 then writefits, patho+'final\I_'+ftime+'.fts', Tsig, header
    if calib eq 1 then writefits, patho+'final\C_'+ftime+'.fts', Tsig, header
    if calib eq 2 then writefits, patho+'final\CI_'+ftime+'.fts', median(Tsig[30/binx:481/binx,*,*,*], dimension=1), header


    if calib then begin
      tlo=-1
      thi=1
    endif else begin
      tlo=-0.03
      thi=0.03
    endelse

    mt=median(Isig) & stdt=2.*stddev(Isig)

    if i eq is then window, 1, xs=1024, ys=1024
    wset, 1

    if calib then begin
      for r=0, 4 do begin
        tv, congrid(bytscl(Isig[*,*,r], mt-stdt, mt+stdt),256/5,512), 0*5+r
        tv, congrid(bytscl(Tsig[*,*,r,1], tlo, thi),256/5,512), 1*5+r
        tv, congrid(bytscl(Tsig[*,*,r,2], tlo, thi),256/5,512), 2*5+r
	tv, congrid(bytscl(Tsig[*,*,r,3], tlo, thi),256/5,512), 3*5+r
      endfor
    endif else begin
        tv, rebin(bytscl(Isig[*,*,0], mt-stdt, mt+stdt),256,512), 0
        tv, rebin(bytscl(Tsig[*,*,0,1], tlo, thi),256,512), 1
        tv, rebin(bytscl(Tsig[*,*,0,2], tlo, thi),256,512), 2
        tv, rebin(bytscl(Tsig[*,*,0,3], tlo, thi),256,512), 3
        tv, rebin(bytscl(Isig[*,*,nw/4], mt-stdt, mt+stdt),256,512), 4
        tv, rebin(bytscl(Tsig[*,*,nw/4,1], tlo, thi),256,512), 5
        tv, rebin(bytscl(Tsig[*,*,nw/4,2], tlo, thi),256,512), 6
        tv, rebin(bytscl(Tsig[*,*,nw/4,3], tlo, thi),256,512), 7
    endelse

    png_canvas=fltarr(1024,1024)
    png_canvas[0:255,512:1023]=rebin(bytscl(Isig[*,*,0], mt-stdt, mt+stdt),256,512)
    png_canvas[256:511,512:1023]=rebin(bytscl(Tsig[*,*,0,1], tlo, thi),256,512)
    png_canvas[512:767,512:1023]=rebin(bytscl(Tsig[*,*,0,2], tlo, thi),256,512)
    png_canvas[768:1023,512:1023]=rebin(bytscl(Tsig[*,*,0,3], tlo, thi),256,512)
    png_canvas[0:255,0:511]=rebin(bytscl(Isig[*,*,nw/4], mt-stdt, mt+stdt),256,512)
    png_canvas[256:511,0:511]=rebin(bytscl(Tsig[*,*,nw/4,1], tlo, thi),256,512)
    png_canvas[512:767,0:511]=rebin(bytscl(Tsig[*,*,nw/4,2], tlo, thi),256,512)
    png_canvas[768:1023,0:511]=rebin(bytscl(Tsig[*,*,nw/4,3], tlo, thi),256,512)

    if calib eq 0 then write_png, patho+'final\I_'+ftime+'.png', png_canvas
    if calib eq 1 then write_png, patho+'final\C_'+ftime+'.png', png_canvas
    if calib eq 2 then write_png, patho+'final\CI_'+ftime+'.png', png_canvas

if i eq is then begin
tf=systime(/seconds)
print, 'A loop done at '+string(tf-ta)
endif

wset, 0


JUMP2:
  endfor

  !p.multi=[0,4,4]

endif


return, 1
!p.multi=[0,1,1]


end
